# -*- coding: utf-8 -*-
import click
import logging
from pathlib import Path
from dotenv import find_dotenv, load_dotenv
import pandas as pd
import pickle
from sklearn.preprocessing import StandardScaler
from sklearn.preprocessing import LabelEncoder
import bisect
import numpy as np


def encode_dataset(input_filepath, output_filepath, encoder_filepath, is_create_encoders):
    """ Encode the dataset: encoding, scaling and filling the
        continuous and categorical values.
    """
    logger = logging.getLogger(__name__)

    FILLERS_PATH = Path.cwd().joinpath(encoder_filepath).joinpath('numerical-fillers.pickle')
    SCALERS_PATH = Path.cwd().joinpath(encoder_filepath).joinpath('numerical-scalers.pickle')
    ENCODERS_PATH = Path.cwd().joinpath(encoder_filepath).joinpath('categorical-encoders.pickle')

    logger.info(f'Encoding file {input_filepath}')
    df = pd.read_feather(Path.cwd().joinpath(input_filepath))

    metadata_cols = ['encounter_id']

    numerical_cols = df.select_dtypes(include='number').columns
    numerical_cols = [x for x in numerical_cols if x not in metadata_cols]
    logger.info(f'Enumerated numerical columns {numerical_cols}')

    categorical_cols = df.select_dtypes(include='category').columns
    categorical_cols = [x for x in categorical_cols if x not in metadata_cols]
    logger.info(f'Enumerated categorical columns {categorical_cols}')

    logger.info(f'Importing fillers, scalers and encoders')

    try:
        fillers = read_obj(FILLERS_PATH)
        scalers = read_obj(SCALERS_PATH)
        encoders = read_obj(ENCODERS_PATH)
    except FileNotFoundError:
        logger.warn('Fillers, scalers or encoders not found.')

    if is_create_encoders:
        logger.info(f'Creating new fillers, scalers, and encoders')
        tmp = df.copy()
        _, fillers = fill(tmp, numerical_cols)
        _, scalers = normalize(tmp, numerical_cols)
        _, encoders = labelencode(tmp, categorical_cols)

    logger.info(f'Filling numericals')
    df, _ = fill(df, numerical_cols, fillers)

    logger.info(f'Normalizing numericals')
    df, _ = normalize(df, numerical_cols, scalers)

    logger.info(f'Labelencoding categoricals')
    df, _ = labelencode(df, categorical_cols, encoders)

    if is_create_encoders:
        logger.info(f'Persisting fillers, scalers and encoders')
        pickle_obj(FILLERS_PATH, fillers)
        pickle_obj(SCALERS_PATH, scalers)
        pickle_obj(ENCODERS_PATH, encoders)

    logger.info(f'Persisting encoded dataset')
    output_filename = Path.cwd().joinpath(input_filepath).stem
    df.reset_index(drop=True).to_feather(Path.cwd().joinpath(output_filepath).joinpath(output_filename + '_encoded.feather'))


def pickle_obj(path, obj):
    with open(path, 'wb') as f:
        pickle.dump(obj, f)


def read_obj(path):
    with open(path, 'rb') as f:
        return pickle.load(f)
    return None


def fill(df, cols, fillers=None, is_demarc=True, fill_val=None):
    if None is fillers:
        fillers = dict()
    for col in cols:
        if col not in fillers:
            if fill_val is not None:
                fillers[col] = fill_val
            else:
                fillers[col] = df[col].dropna().mean()

        if is_demarc:
            df[f'{col}_na'] = pd.isnull(df[col])
        df[col] = df[col].fillna(fillers[col])

    return df, fillers


def normalize(df, cols, scalers=None):
    if None is scalers:
        scalers = dict()

    for col in cols:
        if col not in scalers:
            scalers[col] = StandardScaler(with_mean=True, with_std=True)
            scalers[col].fit(df[col].values.reshape(-1, 1))

        scaler = scalers[col]
        df[col] = scaler.transform(df[col].values.reshape(-1, 1))
    return df, scalers


def labelencode(df, cols, encoders=None, unknown_value='UNK'):
    if None is encoders:
        encoders = dict()

    for col in cols:
        if col not in encoders:
            le = LabelEncoder()
            le.fit(df[col].values)

            # add unknown val to cats
            cats = le.classes_.tolist()
            bisect.insort_left(cats, unknown_value)

            # redefine cats on le
            le.classes_ = np.asarray(cats)

            encoders[col] = le

        le = encoders[col]
        df[col] = df[col].map(lambda x: unknown_value if x not in le.classes_ else x)
        df[col] = le.transform(df[col].values)

    return df, encoders


@click.command()
@click.argument('input_filepath', type=click.Path(exists=True), default='data/interim/training_v2_train.feather')
@click.argument('output_filepath', type=click.Path(exists=True), default='data/processed/')
@click.argument('encoder_filepath', type=click.Path(exists=True), default='models/')
@click.option('--is-create-encoders', is_flag=True)
def main(input_filepath, output_filepath, encoder_filepath, is_create_encoders):
    encode_dataset(input_filepath, output_filepath, encoder_filepath, is_create_encoders)


if __name__ == '__main__':
    log_fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(level=logging.INFO, format=log_fmt)

    # not used in this stub but often useful for finding various files
    project_dir = Path(__file__).resolve().parents[2]

    # find .env automagically by walking up directories until it's found, then
    # load up the .env entries as environment variables
    load_dotenv(find_dotenv())

    main()



# -*- coding: utf-8 -*-
import click
import logging
from pathlib import Path
from dotenv import find_dotenv, load_dotenv
import pandas as pd
import pickle
from sklearn.preprocessing import StandardScaler
from sklearn.preprocessing import LabelEncoder
import bisect
import numpy as np
from itertools import combinations
# from sklearn import decomposition
# from sklearn.cluster import KMeans
# from sklearn.ensemble import RandomForestClassifier
# from sklearn.neighbors import KNeighborsClassifier
# from catboost import CatBoostRegressor


@click.command()
@click.argument('input_filepath', type=click.Path(exists=True))
@click.argument('output_filepath', type=click.Path())
def main(input_filepath, output_filepath):
    """ Runs data processing scripts to turn raw data from (../interim) into
        cleaned data ready to be analyzed (saved in ../processed).
    """
    logger = logging.getLogger(__name__)

    IS_USE_COMBOS = False
    IS_DROP_CONFOUNDING_COLS = False

    INPUT_DIR = Path.cwd().joinpath(input_filepath)
    logger.info(f'feature-engineering with data in {INPUT_DIR.name}')

    logger.info('reading envs')
    # cols
    BINARY_COLS = Path.cwd().joinpath(input_filepath).joinpath('binary-cols.pickle')
    CATEGORICAL_COLS = Path.cwd().joinpath(input_filepath).joinpath('categorical-cols.pickle')
    CONTINUOUS_COLS = Path.cwd().joinpath(input_filepath).joinpath('continuous-cols.pickle')
    TARGET_COL = Path.cwd().joinpath(input_filepath).joinpath('target-col.pickle')

    BINARY_COLS_OUT = Path.cwd().joinpath(output_filepath).joinpath('binary-cols.pickle')
    CATEGORICAL_COLS_OUT = Path.cwd().joinpath(output_filepath).joinpath('categorical-cols.pickle')
    CONTINUOUS_COLS_OUT = Path.cwd().joinpath(output_filepath).joinpath('continuous-cols.pickle')
    TARGET_COL_OUT = Path.cwd().joinpath(output_filepath).joinpath('target-col.pickle')

    # data
    TRAIN_CSV = Path.cwd().joinpath(input_filepath).joinpath('train.csv')
    VAL_CSV = Path.cwd().joinpath(input_filepath).joinpath('val.csv')
    TEST_CSV = Path.cwd().joinpath(input_filepath).joinpath('test.csv')
    PREDS_CSV = Path.cwd().joinpath(input_filepath).joinpath('preds.csv')

    TRAIN_CSV_OUT = Path.cwd().joinpath(output_filepath).joinpath('train.csv')
    VAL_CSV_OUT = Path.cwd().joinpath(output_filepath).joinpath('val.csv')
    TEST_CSV_OUT = Path.cwd().joinpath(output_filepath).joinpath('test.csv')
    PREDS_CSV_OUT = Path.cwd().joinpath(output_filepath).joinpath('preds.csv')

    # metadata
    BINARY_ENCODERS = Path.cwd().joinpath(output_filepath).joinpath('binary-encoders.pickle')
    CATEGORICAL_ENCODERS = Path.cwd().joinpath(output_filepath).joinpath('categorical-encoders.pickle')
    TARGET_ENCODERS = Path.cwd().joinpath(output_filepath).joinpath('target-encoders.pickle')
    CONTINUOUS_SCALERS = Path.cwd().joinpath(output_filepath).joinpath('continuous-scalers.pickle')
    CONTINUOUS_FILLERS = Path.cwd().joinpath(output_filepath).joinpath('continuous-fillers.pickle')

    logger.info('reading in data')
    binary_cols = read_obj(BINARY_COLS)
    categorical_cols = read_obj(CATEGORICAL_COLS)
    continuous_cols = read_obj(CONTINUOUS_COLS)
    target_col = read_obj(TARGET_COL)

    model_features = ['apache_4a_hospital_death_prob', 'age', 'ventilated_apache', 'apache_3j_diagnosis', 'd1_spo2_min', 'apache_4a_icu_death_prob', 'd1_heartrate_min', 'd1_lactate_min', 'd1_temp_max', 'bmi', 'd1_wbc_min', 'd1_temp_min', 'd1_resprate_min', 'd1_heartrate_max', 'd1_bun_max', 'd1_sodium_max', 'd1_resprate_max', 'd1_bun_min', 'urineoutput_apache', 'd1_sysbp_noninvasive_min', 'h1_resprate_min', 'd1_platelets_min', 'd1_mbp_min', 'resprate_apache', 'gcs_verbal_apache', 'd1_glucose_min', 'hospital_admit_source', 'd1_pao2fio2ratio_max', 'apache_2_diagnosis', 'wbc_apache', 'pre_icu_los_days', 'd1_arterial_ph_min', 'icu_admit_source', 'd1_sysbp_min', 'gcs_motor_apache', 'd1_arterial_po2_max_na'] # noqa
    # model_cat_features = [x for x in model_features if x in categorical_cols or x in binary_cols]
    model_cat_features = [
        'ventilated_apache',
        'apache_3j_diagnosis',
        'bmi',
        'hospital_admit_source',
        'apache_2_diagnosis',
        'icu_admit_source',
        'apache_2_bodysystem',
        'icu_type',
        'apache_3j_bodysystem',
        'solid_tumor_with_metastasis',
        'elective_surgery',
        'immunosuppression',
        'arf_apache',
        'intubated_apache',
        'cirrhosis'
    ]

    train = pd.read_csv(TRAIN_CSV)
    val = pd.read_csv(VAL_CSV)
    test = pd.read_csv(TEST_CSV)
    preds = pd.read_csv(PREDS_CSV)

    # model = CatBoostRegressor(random_seed=42)

    # logger.info('imputing apache_4a_hospital_death_prob')
    # # Fit model
    # tmp = train.copy().dropna(subset=['apache_4a_hospital_death_prob']).fillna(-999)
    # tmp = tmp[tmp['apache_4a_hospital_death_prob'] >= 0]
    # X = tmp[tmp.columns.difference(['apache_4a_hospital_death_prob'])]
    # X[X.columns.difference(categorical_cols+binary_cols)
    #   ] = X[X.columns.difference(categorical_cols+binary_cols)].astype('float32')
    # X[categorical_cols+binary_cols] = X[categorical_cols+binary_cols].astype('str')
    # X.pop(target_col)
    # y = tmp['apache_4a_hospital_death_prob'].astype('float32')
    # model.fit(X, y, cat_features=categorical_cols+binary_cols)

    # tmp2 = train[(0 > train['apache_4a_hospital_death_prob']) | (
    #     np.isnan(train['apache_4a_hospital_death_prob']))].copy().fillna(-999)
    # X2 = tmp2[tmp2.columns.difference(['apache_4a_hospital_death_prob'])]
    # X2[X2.columns.difference(categorical_cols+binary_cols)
    #    ] = X2[X2.columns.difference(categorical_cols+binary_cols)].astype('float32')
    # X2[categorical_cols+binary_cols] = X2[categorical_cols+binary_cols].astype('str')
    # X2.pop(target_col)
    # # y2 = tmp2['apache_4a_hospital_death_prob'].astype('float32')
    # death_prob_preds = pd.DataFrame(model.predict(X2), columns=['apache_4a_hospital_death_prob'])
    # samp = pd.concat(
    #     [
    #         death_prob_preds.reset_index(drop=True),
    #         pd.DataFrame(tmp2['hospital_death'], columns=['hospital_death']).reset_index(drop=True)
    #     ], axis=1)
    # samp = pd.concat([samp.reset_index(drop=True), pd.DataFrame(
    #     X2['encounter_id'], columns=['encounter_id']).reset_index(drop=True)], axis=1)
    # train.loc[
    #     train['encounter_id'].isin(samp['encounter_id']),
    #     'apache_4a_hospital_death_prob'
    # ] = samp.loc[
    #     samp['encounter_id'].isin(train['encounter_id']),
    #     'apache_4a_hospital_death_prob'
    # ].values

    # tmp2 = val[(0 > val['apache_4a_hospital_death_prob']) | (
    #     np.isnan(val['apache_4a_hospital_death_prob']))].copy().fillna(-999)
    # X2 = tmp2[tmp2.columns.difference(['apache_4a_hospital_death_prob'])]
    # X2[X2.columns.difference(categorical_cols+binary_cols)
    #    ] = X2[X2.columns.difference(categorical_cols+binary_cols)].astype('float32')
    # X2[categorical_cols+binary_cols] = X2[categorical_cols+binary_cols].astype('str')
    # X2.pop(target_col)
    # # y2 = tmp2['apache_4a_hospital_death_prob'].astype('float32')
    # death_prob_preds = pd.DataFrame(model.predict(X2), columns=['apache_4a_hospital_death_prob'])
    # samp = pd.concat(
    #     [
    #         death_prob_preds.reset_index(drop=True),
    #         pd.DataFrame(tmp2['hospital_death'], columns=['hospital_death']).reset_index(drop=True)
    #     ], axis=1)
    # samp = pd.concat([samp.reset_index(drop=True), pd.DataFrame(
    #     X2['encounter_id'], columns=['encounter_id']).reset_index(drop=True)], axis=1)
    # val.loc[
    #     val['encounter_id'].isin(samp['encounter_id']),
    #     'apache_4a_hospital_death_prob'
    # ] = samp.loc[
    #     samp['encounter_id'].isin(val['encounter_id']),
    #     'apache_4a_hospital_death_prob'
    # ].values

    # tmp2 = test[(0 > test['apache_4a_hospital_death_prob']) | (
    #     np.isnan(test['apache_4a_hospital_death_prob']))].copy().fillna(-999)
    # X2 = tmp2[tmp2.columns.difference(['apache_4a_hospital_death_prob'])]
    # X2[X2.columns.difference(categorical_cols+binary_cols)
    #    ] = X2[X2.columns.difference(categorical_cols+binary_cols)].astype('float32')
    # X2[categorical_cols+binary_cols] = X2[categorical_cols+binary_cols].astype('str')
    # X2.pop(target_col)
    # # y2 = tmp2['apache_4a_hospital_death_prob'].astype('float32')
    # death_prob_preds = pd.DataFrame(model.predict(X2), columns=['apache_4a_hospital_death_prob'])
    # samp = pd.concat(
    #     [
    #         death_prob_preds.reset_index(drop=True),
    #         pd.DataFrame(tmp2['hospital_death'], columns=['hospital_death']).reset_index(drop=True)
    #     ], axis=1)
    # samp = pd.concat([samp.reset_index(drop=True), pd.DataFrame(
    #     X2['encounter_id'], columns=['encounter_id']).reset_index(drop=True)], axis=1)
    # test.loc[
    #     test['encounter_id'].isin(samp['encounter_id']),
    #     'apache_4a_hospital_death_prob'
    # ] = samp.loc[
    #     samp['encounter_id'].isin(test['encounter_id']),
    #     'apache_4a_hospital_death_prob'
    # ].values

    # tmp2 = preds[(0 > preds['apache_4a_hospital_death_prob']) | (
    #     np.isnan(preds['apache_4a_hospital_death_prob']))].copy().fillna(-999)
    # X2 = tmp2[tmp2.columns.difference(['apache_4a_hospital_death_prob'])]
    # X2[X2.columns.difference(categorical_cols+binary_cols)
    #    ] = X2[X2.columns.difference(categorical_cols+binary_cols)].astype('float32')
    # X2[categorical_cols+binary_cols] = X2[categorical_cols+binary_cols].astype('str')
    # # y2 = tmp2['apache_4a_hospital_death_prob'].astype('float32')
    # death_prob_preds = pd.DataFrame(model.predict(X2[model.feature_names_]), columns=['apache_4a_hospital_death_prob'])
    # samp = death_prob_preds.reset_index(drop=True)
    # samp = pd.concat([samp.reset_index(drop=True), pd.DataFrame(
    #     X2['encounter_id'], columns=['encounter_id']).reset_index(drop=True)], axis=1)
    # preds.loc[
    #     preds['encounter_id'].isin(samp['encounter_id']),
    #     'apache_4a_hospital_death_prob'
    # ] = samp.loc[
    #     samp['encounter_id'].isin(preds['encounter_id']),
    #     'apache_4a_hospital_death_prob'
    # ].values

    # logger.info('forming distributional clusters')
    # pca = decomposition.PCA(n_components=4)
    # clustering_cols = [
    #     'd1_spo2_min',
    #     'd1_bun_min',
    #     'gcs_motor_apache',
    #     'd1_bun_max',
    #     'd1_resprate_max',
    #     'apache_3j_diagnosis',
    #     'd1_heartrate_max'
    # ]
    # #  clsutering_cols = [continuous_cols]
    # cl_train = train[clustering_cols].copy().astype('float32')
    # cl_val = val[clustering_cols].copy().astype('float32')
    # cl_test = test[clustering_cols].copy().astype('float32')
    # cl_preds = preds[clustering_cols].copy().astype('float32')

    # cl_train, cluster_fillers = fill(cl_train, clustering_cols, is_demarc=False)
    # cl_val, _ = fill(cl_val, clustering_cols, cluster_fillers, is_demarc=False)
    # cl_test, _ = fill(cl_test, clustering_cols, cluster_fillers, is_demarc=False)
    # cl_preds, _ = fill(cl_preds, clustering_cols, cluster_fillers, is_demarc=False)

    # pca.fit(cl_train)
    # cl_train = pca.transform(cl_train)
    # cl_val = pca.transform(cl_val)
    # cl_test = pca.transform(cl_test)
    # cl_preds = pca.transform(cl_preds)

    # when d1_heartrate_max is -1000 and d1_platelets_min is [0, 500] -> not dead
    # when d1_wbc_min is -1000 and d1_platelets_min [250, 500] -> not dead
    # more often than not when d1_lactate_min is -1000, then it is not dead, even otherwise
    # more often than not when d1_arterial_ph_min is -1000, then it is not dead,  even otherwise

    # kmeans = KMeans(n_clusters=2, random_state=0).fit(cl_train)
    # train = pd.concat([train, pd.DataFrame(kmeans.predict(cl_train), columns=['cluster_id'])], axis=1)
    # val = pd.concat([val, pd.DataFrame(kmeans.predict(cl_val), columns=['cluster_id'])], axis=1)
    # test = pd.concat([test, pd.DataFrame(kmeans.predict(cl_test), columns=['cluster_id'])], axis=1)
    # preds = pd.concat([preds, pd.DataFrame(kmeans.predict(cl_preds), columns=['cluster_id'])], axis=1)
    # categorical_cols.append('cluster_id')

    logger.info('converting height units to meters')
    train['height'] = train['height']/100
    val['height'] = val['height']/100
    test['height'] = test['height']/100
    preds['height'] = preds['height']/100

    logger.info('recalculating bmi')
    train['bmi'] = train['weight']/(train['height']**2)
    val['bmi'] = val['weight']/(val['height']**2)
    test['bmi'] = test['weight']/(test['height']**2)
    preds['bmi'] = preds['weight']/(preds['height']**2)

    logger.info('creating new feature weightclass')
    train['weightclass'] = train['bmi'].map(weighted_class)
    val['weightclass'] = val['bmi'].map(weighted_class)
    test['weightclass'] = test['bmi'].map(weighted_class)
    preds['weightclass'] = preds['bmi'].map(weighted_class)
    categorical_cols.append('weightclass')

    logger.info('adding pre_icu_los_days_is_bad marker')
    train['pre_icu_los_days_is_bad'] = 0 > train['pre_icu_los_days']
    val['pre_icu_los_days_is_bad'] = 0 > val['pre_icu_los_days']
    test['pre_icu_los_days_is_bad'] = 0 > test['pre_icu_los_days']
    preds['pre_icu_los_days_is_bad'] = 0 > preds['pre_icu_los_days']
    categorical_cols.append('pre_icu_los_days_is_bad')

    logger.info('adding apache_4a_hospital_death_prob_is_bad')
    train['apache_4a_hospital_death_prob_is_bad'] = 0 > train['apache_4a_hospital_death_prob']
    val['apache_4a_hospital_death_prob_is_bad'] = 0 > val['apache_4a_hospital_death_prob']
    test['apache_4a_hospital_death_prob_is_bad'] = 0 > test['apache_4a_hospital_death_prob']
    preds['apache_4a_hospital_death_prob_is_bad'] = 0 > preds['apache_4a_hospital_death_prob']
    categorical_cols.append('apache_4a_hospital_death_prob_is_bad')

    logger.info('adding apache_4a_icu_death_prob_is_bad')
    train['apache_4a_icu_death_prob_is_bad'] = 0 > train['apache_4a_icu_death_prob']
    val['apache_4a_icu_death_prob_is_bad'] = 0 > val['apache_4a_icu_death_prob']
    test['apache_4a_icu_death_prob_is_bad'] = 0 > test['apache_4a_icu_death_prob']
    preds['apache_4a_icu_death_prob_is_bad'] = 0 > preds['apache_4a_icu_death_prob']
    categorical_cols.append('apache_4a_icu_death_prob_is_bad')

    # logger.info('adding d1_heartrate_min_d1_temp_max_tell')
    # train['d1_heartrate_min_d1_temp_max_tell'] = np.isnan(train['d1_heartrate_min']) & np.isnan(train['d1_temp_max'])
    # val['d1_heartrate_min_d1_temp_max_tell'] = np.isnan(val['d1_heartrate_min']) & np.isnan(val['d1_temp_max'])
    # test['d1_heartrate_min_d1_temp_max_tell'] = np.isnan(test['d1_heartrate_min']) & np.isnan(test['d1_temp_max'])
    # preds['d1_heartrate_min_d1_temp_max_tell'] = np.isnan(preds['d1_heartrate_min']) & np.isnan(preds['d1_temp_max'])
    # categorical_cols.append('d1_heartrate_min_d1_temp_max_tell_1')

    # top_continuous_cols = list(set([
    #     'apache_4a_hospital_death_prob',
    #     'age',
    #     'apache_4a_icu_death_prob',
    #     'd1_spo2_min',
    #     'd1_bun_min',
    #     'd1_heartrate_min',
    #     'd1_resprate_max',
    #     'd1_resprate_min',
    #     'd1_wbc_min',
    #     'd1_temp_max',
    #     'd1_heartrate_max',
    #     'd1_temp_min',
    #     'd1_sodium_max',
    #     'd1_lactate_min',
    #     'd1_bun_max',
    #     'd1_glucose_min',
    #     'd1_platelets_min',
    #     'd1_arterial_ph_min',
    #     'd1_mbp_noninvasive_min',
    #     'pre_icu_los_days',
    #     'd1_sysbp_min',
    #     'wbc_apache',
    #     'd1_mbp_min',
    #     'bun_apache',
    #     'd1_sysbp_noninvasive_min',
    #     'gcs_verbal_apache',
    #     'resprate_apache'
    # ]))
    # top_continuous_cols = [
    #     ('d1_glucose_min', 'd1_platelets_min'),
    #     ('d1_bun_max', 'd1_platelets_min'),
    #     ('d1_platelets_min', 'd1_sodium_max'),
    #     ('d1_glucose_min', 'd1_heartrate_max'),
    #     ('d1_platelets_min', 'd1_heartrate_max'),
    #     ('d1_bun_max', 'd1_heartrate_max'),
    #     ('d1_lactate_min', 'd1_heartrate_max'),
    #     ('d1_platelets_min', 'd1_wbc_min'),
    #     ('pre_icu_los_days', 'resprate_apache'),
    #     ('pre_icu_los_days', 'gcs_verbal_apache'),
    #     ('pre_icu_los_days', 'bun_apache'),
    #     ('pre_icu_los_days', 'd1_mbp_noninvasive_min'),
    #     ('pre_icu_los_days', 'd1_sysbp_min'),
    #     ('pre_icu_los_days', 'd1_arterial_ph_min'),
    #     ('d1_mbp_min', 'd1_mbp_noninvasive_min'),
    #     ('d1_temp_max', 'd1_heartrate_min'),
    #     ('age', 'd1_resprate_min'),

    #     ('d1_bun_max', 'd1_glucose_min')
    # ]
    # for feature_a, feature_b in list(combinations(top_continuous_cols, 2)):
    #     # rfr = RandomForestClassifier(random_state=0)
    #     rfr = KNeighborsClassifier(n_neighbors=5, weights='distance')

    #     cols = [feature_a, feature_b]
    #     feature_name = f'tell_{feature_a}_{feature_b}'
    #     data = train[cols].fillna(-999)
    #     rfr.fit(data, train[target_col])

    #     logger.info(f'classifying {cols}')
    #     classified_train = pd.DataFrame(rfr.predict_proba(train[cols].fillna(-999))[:, 1], columns=[feature_name])
    #     classified_val = pd.DataFrame(rfr.predict_proba(val[cols].fillna(-999))[:, 1], columns=[feature_name])
    #     classified_test = pd.DataFrame(rfr.predict_proba(test[cols].fillna(-999))[:, 1], columns=[feature_name])
    #     classified_preds = pd.DataFrame(rfr.predict_proba(preds[cols].fillna(-999))[:, 1], columns=[feature_name])
    #     logger.info(f'{classified_train.shape}, {classified_val.shape}, {classified_test.shape}, {classified_preds.shape},')

    #     train = pd.concat([train, classified_train], axis=1)
    #     val = pd.concat([val, classified_val], axis=1)
    #     test = pd.concat([test, classified_test], axis=1)
    #     preds = pd.concat([preds, classified_preds], axis=1)
    #     continuous_cols.append(feature_name)

    # aggregate icu
    train['hospital_admit_source_is_icu'] = train['hospital_admit_source'].apply(
        lambda x:
        'True' if x in [
            'Other ICU',
            'ICU to SDU',
            'ICU'
        ] else 'False')
    val['hospital_admit_source_is_icu'] = val['hospital_admit_source'].apply(
        lambda x:
        'True' if x in [
            'Other ICU',
            'ICU to SDU',
            'ICU'
        ] else 'False')
    test['hospital_admit_source_is_icu'] = test['hospital_admit_source'].apply(
        lambda x:
        'True' if x in [
            'Other ICU',
            'ICU to SDU',
            'ICU'
        ] else 'False')
    preds['hospital_admit_source_is_icu'] = preds['hospital_admit_source'].apply(
        lambda x:
            'True' if x in [
                'Other ICU',
                'ICU to SDU',
                'ICU'
            ] else 'False')
    categorical_cols.append('hospital_admit_source_is_icu')

    # aggregate ethnicity
    common_cols = [np.nan, 'Other/Unknown']
    train['ethnicity_is_unknown'] = train['ethnicity'].apply(lambda x: True if x in common_cols else False)
    val['ethnicity_is_unknown'] = val['ethnicity'].apply(lambda x: True if x in common_cols else False)
    test['ethnicity_is_unknown'] = test['ethnicity'].apply(lambda x: True if x in common_cols else False)
    preds['ethnicity_is_unknown'] = preds['ethnicity'].apply(lambda x: True if x in common_cols else False)
    categorical_cols.append('ethnicity_is_unknown')

    # aggregate cardiac
    common_cols = ['CTICU', 'CCU-CTICU', 'Cardiac ICU', 'CSICU']
    train['icu_type_is_cardiac'] = train['icu_type'].apply(lambda x: True if x in common_cols else False)
    val['icu_type_is_cardiac'] = val['icu_type'].apply(lambda x: True if x in common_cols else False)
    test['icu_type_is_cardiac'] = test['icu_type'].apply(lambda x: True if x in common_cols else False)
    preds['icu_type_is_cardiac'] = preds['icu_type'].apply(lambda x: True if x in common_cols else False)
    categorical_cols.append('icu_type_is_cardiac')

    # aggregate apache_2_bodysystem
    common_cols = ['Undefined Diagnoses', np.nan, 'Undefined diagnoses']
    train['apache_2_bodysystem_is_undefined'] = train['apache_2_bodysystem'].apply(
        lambda x: True if x in common_cols else False)
    val['apache_2_bodysystem_is_undefined'] = val['apache_2_bodysystem'].apply(lambda x: True if x in common_cols else False)
    test['apache_2_bodysystem_is_undefined'] = test['apache_2_bodysystem'].apply(lambda x: True if x in common_cols else False)
    preds['apache_2_bodysystem_is_undefined'] = preds['apache_2_bodysystem'].apply(
        lambda x: True if x in common_cols else False)
    categorical_cols.append('apache_2_bodysystem_is_undefined')

    if IS_USE_COMBOS:

        logger.info('synthesizing categorical pairs')
        # pair_cols = [
        #     'ethnicity',
        #     'gender',
        #     'hospital_admit_source',
        #     'icu_admit_source',
        #     'icu_stay_type',
        #     'icu_type',
        #     'apache_3j_bodysystem',
        #     'apache_2_bodysystem'
        # ]
        # cmbs = list(combinations(pair_cols, 2))
        cmbs = list(combinations(model_cat_features, 2))
        combo_cols = list()
        for cols in cmbs:
            col_name = f'paired_{"_".join(cols)}'
            combo_cols.append(col_name)
            train[col_name] = concat_columns(train, cols)
            val[col_name] = concat_columns(val, cols)
            test[col_name] = concat_columns(test, cols)
            preds[col_name] = concat_columns(preds, cols)
        categorical_cols.extend(combo_cols)

        logger.info('synthesizing categorical triplets')
        # cmbs = list(combinations(pair_cols, 3))
        cmbs = list(combinations(model_cat_features, 3))
        combo_cols = list()
        for cols in cmbs:
            col_name = f'threed_{"_".join(cols)}'
            combo_cols.append(col_name)
            train[col_name] = concat_columns(train, cols)
            val[col_name] = concat_columns(val, cols)
            test[col_name] = concat_columns(test, cols)
            preds[col_name] = concat_columns(preds, cols)
        categorical_cols.extend(combo_cols)

        # logger.info('synthesizing binary pairs')
        # combo_cols = list()
        # for cols in list(combinations(binary_cols, 2)):
        #     col_name = f'paired_binary_{"_".join(cols)}'
        #     combo_cols.append(col_name)
        #     train[col_name] = concat_columns(train, cols)
        #     val[col_name] = concat_columns(val, cols)
        #     test[col_name] = concat_columns(test, cols)
        #     preds[col_name] = concat_columns(preds, cols)
        # categorical_cols.extend(combo_cols)

    logger.info('typifying...')
    # continuous
    train[continuous_cols] = train[continuous_cols].astype('float32')
    val[continuous_cols] = val[continuous_cols].astype('float32')
    test[continuous_cols] = test[continuous_cols].astype('float32')
    preds[continuous_cols] = preds[continuous_cols].astype('float32')

    # cats
    train[categorical_cols] = train[categorical_cols].astype('str').astype('category')
    val[categorical_cols] = val[categorical_cols].astype('str').astype('category')
    test[categorical_cols] = test[categorical_cols].astype('str').astype('category')
    preds[categorical_cols] = preds[categorical_cols].astype('str').astype('category')

    # binary
    train[binary_cols] = train[binary_cols].astype('str').astype('category')
    val[binary_cols] = val[binary_cols].astype('str').astype('category')
    test[binary_cols] = test[binary_cols].astype('str').astype('category')
    preds[binary_cols] = preds[binary_cols].astype('str').astype('category')

    # targets
    train[target_col] = train[target_col].astype('str').astype('category')
    val[target_col] = val[target_col].astype('str').astype('category')
    test[target_col] = test[target_col].astype('str').astype('category')

    logger.info("dropping")
    train = train.dropna(how='all')
    val = val.dropna(how='all')
    test = test.dropna(how='all')
    preds = preds.dropna(how='all')

    if IS_DROP_CONFOUNDING_COLS:
        logger.info('dropping confounding features')
        confounding_cols = ['apache_4a_hospital_death_prob', 'apache_4a_icu_death_prob']

        # remove confounding vars - biases and undue variance
        for x in confounding_cols:
            continuous_cols.remove(x)
        train = train.drop(columns=confounding_cols)
        val = val.drop(columns=confounding_cols)
        test = test.drop(columns=confounding_cols)
        preds = preds.drop(columns=confounding_cols)

    train = train[[target_col] + continuous_cols + categorical_cols + binary_cols]
    val = val[[target_col] + continuous_cols + categorical_cols + binary_cols]
    test = test[[target_col] + continuous_cols + categorical_cols + binary_cols]
    preds = preds[continuous_cols + categorical_cols + binary_cols]

    # keep = ['encounter_id']
    # model_features = ['apache_4a_hospital_death_prob', 'age', 'apache_3j_diagnosis', 'd1_heartrate_min', 'bmi', 'apache_4a_icu_death_prob', 'd1_spo2_min', 'd1_bun_min', 'ventilated_apache', 'd1_temp_min', 'paired_ventilated_apache_icu_admit_source', 'd1_lactate_min', 'd1_bun_max', 'd1_platelets_min', 'd1_wbc_min', 'd1_sodium_max', 'd1_resprate_min', 'd1_resprate_max', 'urineoutput_apache', 'd1_arterial_ph_min', 'd1_temp_max', 'd1_mbp_min', 'apache_2_diagnosis', 'd1_sysbp_noninvasive_min', 'd1_heartrate_max', 'd1_sysbp_min', 'gcs_verbal_apache', 'resprate_apache', 'd1_pao2fio2ratio_max', 'd1_glucose_min', 'gcs_motor_apache', 'pre_icu_los_days', 'wbc_apache_na', 'h1_resprate_min', 'hospital_admit_source', 'wbc_apache', 'icu_admit_source', 'd1_arterial_ph_min_na', 'urineoutput_apache_na', 'paired_ventilated_apache_hospital_admit_source'] # noqa
    # model_features = ['apache_4a_hospital_death_prob', 'age', 'apache_3j_diagnosis', 'ventilated_apache', 'd1_spo2_min', 'apache_4a_icu_death_prob', 'd1_heartrate_min', 'd1_bun_min', 'bmi', 'd1_wbc_min', 'd1_heartrate_max', 'd1_resprate_max', 'd1_temp_max', 'd1_temp_min', 'd1_resprate_min', 'd1_sodium_max', 'gcs_verbal_apache', 'd1_lactate_min', 'd1_bun_max', 'd1_glucose_min', 'h1_resprate_min', 'd1_platelets_min', 'd1_mbp_min', 'hospital_admit_source', 'h1_heartrate_max', 'd1_arterial_ph_min', 'apache_3j_bodysystem', 'gcs_eyes_apache', 'h1_sysbp_min', 'bun_apache', 'apache_2_diagnosis', 'resprate_apache', 'd1_sysbp_min', 'pre_icu_los_days', 'd1_glucose_max', 'd1_creatinine_max', 'd1_arterial_pco2_max_na', 'd1_hco3_max', 'd1_hemaglobin_max', 'urineoutput_apache_na', 'd1_pao2fio2ratio_max', 'icu_type', 'wbc_apache', 'd1_sysbp_noninvasive_min', 'h1_diasbp_max', 'd1_lactate_max', 'gcs_motor_apache', 'd1_mbp_noninvasive_min', 'icu_admit_source', 'd1_arterial_pco2_max', 'd1_arterial_ph_max', 'map_apache', 'h1_mbp_noninvasive_min', 'bilirubin_apache', 'urineoutput_apache', 'h1_heartrate_min', 'd1_bilirubin_min', 'd1_albumin_max', 'd1_sysbp_noninvasive_max', 'h1_temp_max', 'height', 'd1_potassium_min', 'heart_rate_apache', 'd1_hemaglobin_min', 'd1_sysbp_max', 'd1_albumin_min', 'weight', 'glucose_apache', 'paco2_apache_na', 'd1_pao2fio2ratio_min', 'h1_temp_min', 'd1_platelets_max', 'age_na', 'd1_arterial_pco2_min', 'h1_resprate_max', 'd1_calcium_max', 'elective_surgery', 'd1_hematocrit_min', 'temp_apache', 'd1_spo2_max', 'weightclass', 'h1_mbp_min', 'd1_creatinine_min', 'solid_tumor_with_metastasis', 'd1_arterial_po2_min', 'd1_hco3_min', 'paco2_apache', 'd1_hematocrit_max', 'd1_calcium_min', 'd1_inr_min', 'd1_wbc_max', 'hematocrit_apache', 'h1_sysbp_invasive_max', 'sodium_apache', 'd1_inr_max', 'h1_spo2_min', 'h1_diasbp_min', 'h1_inr_max', 'h1_sysbp_noninvasive_max', 'd1_bilirubin_max', 'd1_sysbp_invasive_min', 'h1_diasbp_noninvasive_min', 'd1_diasbp_min', 'apache_4a_hospital_death_prob_is_bad', 'd1_arterial_po2_max', 'creatinine_apache', 'd1_diasbp_invasive_min', 'd1_potassium_max_na', 'pao2_apache', 'h1_glucose_min', 'creatinine_apache_na', 'ph_apache', 'd1_mbp_invasive_min', 'h1_diasbp_noninvasive_max', 'd1_pao2fio2ratio_min_na', 'h1_wbc_max', 'h1_sysbp_max', 'glucose_apache_na', 'h1_mbp_noninvasive_max', 'd1_sodium_min'] # noqa
    # categorical_cols = [x for x in categorical_cols if x in model_features or x in keep]
    # binary_cols = [x for x in binary_cols if x in model_features or x in keep]
    # continuous_cols = [x for x in continuous_cols if x in model_features or x in keep]

    logger.info('dropping identifiers from list cols')
    drop_cols = ['encounter_id', 'hospital_id', 'patient_id', 'icu_id']  # , 'ethnicity', 'readmission_status']
    for col in drop_cols:
        if col in continuous_cols:
            continuous_cols.remove(col)
        if col in categorical_cols:
            categorical_cols.remove(col)
        if col in binary_cols:
            binary_cols.remove(col)

    logger.info('filling')
    train, fillers = fill(train, continuous_cols)
    val, _ = fill(val, continuous_cols, fillers)
    test, _ = fill(test, continuous_cols, fillers)
    preds, _ = fill(preds, continuous_cols, fillers)
    # train, fillers = fill_by_cluster(train, continuous_cols)
    # val, _ = fill_by_cluster(val, continuous_cols, fillers)
    # test, _ = fill_by_cluster(test, continuous_cols, fillers)
    # preds, _ = fill_by_cluster(preds, continuous_cols, fillers)
    categorical_cols.extend([f'{x}_na' for x in continuous_cols])

    logger.info('normalizing')
    train, scalers = normalize(train, continuous_cols)
    val, _ = normalize(val, continuous_cols, scalers)
    test, _ = normalize(test, continuous_cols, scalers)
    preds, _ = normalize(preds, continuous_cols, scalers)

    logger.info('label encoding categoricals')

    # to handle converting fill_nas from bool to str
    train[categorical_cols] = train[categorical_cols].astype('str').astype('category')
    val[categorical_cols] = val[categorical_cols].astype('str').astype('category')
    test[categorical_cols] = test[categorical_cols].astype('str').astype('category')
    preds[categorical_cols] = preds[categorical_cols].astype('str').astype('category')

    train, label_encoders = labelencode(train, categorical_cols)
    val, _ = labelencode(val, categorical_cols, label_encoders)
    test, _ = labelencode(test, categorical_cols, label_encoders)
    preds, _ = labelencode(preds, categorical_cols, label_encoders)

    logger.info('label encoding targets')
    train, target_encoders = labelencode(train, [target_col])
    val, _ = labelencode(val, [target_col], target_encoders)
    test, _ = labelencode(test, [target_col], target_encoders)

    logger.info('one hot encoding categoricals')
    # TODO: change to ohe later
    train, ohe_encoders = labelencode(train, binary_cols)
    val, _ = labelencode(val, binary_cols, ohe_encoders)
    test, _ = labelencode(test, binary_cols, ohe_encoders)
    preds, _ = labelencode(preds, binary_cols, ohe_encoders)

    logger.info('persisting data')
    # cols
    pickle_obj(BINARY_COLS_OUT, binary_cols)
    pickle_obj(CATEGORICAL_COLS_OUT, categorical_cols)
    pickle_obj(CONTINUOUS_COLS_OUT, continuous_cols)
    pickle_obj(TARGET_COL_OUT, target_col)

    # metadata
    pickle_obj(BINARY_ENCODERS, ohe_encoders)
    pickle_obj(CATEGORICAL_ENCODERS, label_encoders)
    pickle_obj(TARGET_ENCODERS, target_encoders)
    pickle_obj(CONTINUOUS_SCALERS, scalers)
    pickle_obj(CONTINUOUS_FILLERS, fillers)

    # data
    train.to_csv(TRAIN_CSV_OUT, index=False)
    val.to_csv(VAL_CSV_OUT, index=False)
    test.to_csv(TEST_CSV_OUT, index=False)
    preds.to_csv(PREDS_CSV_OUT, index=False)


def read_obj(path):
    with open(path, 'rb') as f:
        return pickle.load(f)
    return None


def concat_columns(df, columns):
    value = df[columns[0]].astype(str) + ' '
    for col in columns[1:]:
        value += df[col].astype(str) + ' '
    return value


def weighted_class(x):
    if pd.isna(x):
        return np.nan
    elif x < 15:
        return 'very severely underweight'
    elif x >= 15 and x < 16:
        return 'severely weight'
    elif x >= 16 and x < 18.5:
        return 'underweight'
    elif x >= 18.5 and x < 25:
        return 'healthy weight'
    elif x >= 25 and x < 30:
        return 'overweight'
    elif x >= 30 and x < 35:
        return 'class 1'
    elif x >= 35 and x < 40:
        return 'class 2'
    else:
        return 'class 3'
# def fill(df, cols, fillers=None):
#     if None is fillers:
#         fillers = dict()

#         for col in cols:
#             logging.info(f'generating fill col {col}')
#             for hospital_id in df['hospital_id'].unique():

#                 subset = df[df['hospital_id'] == hospital_id]
#                 hospital_col_key = f'{col}_{hospital_id}'

#                 if hospital_col_key not in fillers:
#                     fillers[hospital_col_key] = subset[col].dropna().median()

#             fillers[col] = df[col].dropna().median()

#     for col in cols:
#         logging.info(f'fillling {col}')
#         for hospital_id in df['hospital_id'].unique():
#             subset = df[df['hospital_id'] == hospital_id]
#             hospital_col_key = f'{col}_{hospital_id}'

#             if hospital_col_key in fillers:
#                 df.loc[df['hospital_id'] == hospital_id, col] = subset[col].fillna(fillers[hospital_col_key])
#             else:
#                 df.loc[df['hospital_id'] == hospital_id, col] = subset[col].fillna(fillers[col])

#             df.loc[df['hospital_id'] == hospital_id, f'{col}_na'] = pd.isnull(subset[col])

#     return df, fillers


def fill_by_cluster(df, cols, fillers=None):
    if None is fillers:
        fillers = dict()

        for col in cols:
            logging.info(f'generating fill col {col}')
            for hospital_id in df['cluster_id'].unique():

                subset = df[df['cluster_id'] == hospital_id]
                hospital_col_key = f'{col}_{hospital_id}'

                if hospital_col_key not in fillers:
                    fillers[hospital_col_key] = subset[col].dropna().mean()

            fillers[col] = df[col].dropna().mean()

    for col in cols:
        logging.info(f'fillling {col}')
        for hospital_id in df['cluster_id'].unique():
            subset = df[df['cluster_id'] == hospital_id]
            hospital_col_key = f'{col}_{hospital_id}'

            if hospital_col_key in fillers:
                df.loc[df['cluster_id'] == hospital_id, col] = subset[col].fillna(fillers[hospital_col_key])
            else:
                df.loc[df['cluster_id'] == hospital_id, col] = subset[col].fillna(fillers[col])

            df.loc[df['cluster_id'] == hospital_id, f'{col}_na'] = pd.isnull(subset[col])

    return df, fillers


def fill(df, cols, fillers=None, is_demarc=True, fill_val=None):
    if None is fillers:
        fillers = dict()
    for col in cols:
        if col not in fillers:
            if fill_val is not None:
                fillers[col] = -999
            else:
                fillers[col] = df[col].dropna().mean()

        if is_demarc:
            df[f'{col}_na'] = pd.isnull(df[col])
        df[col] = df[col].fillna(fillers[col])

    return df, fillers


def normalize(df, cols, scalers=None):
    if None is scalers:
        scalers = dict()

    for col in cols:
        if col not in scalers:
            scalers[col] = StandardScaler(with_mean=True, with_std=True)
            scalers[col].fit(df[col].values.reshape(-1, 1))

        scaler = scalers[col]
        df[col] = scaler.transform(df[col].values.reshape(-1, 1))
    return df, scalers


def labelencode(df, cols, encoders=None, unknown_value='UNK'):
    if None is encoders:
        encoders = dict()

    for col in cols:
        if col not in encoders:
            le = LabelEncoder()
            le.fit(df[col].values)

            # add unknown val to cats
            cats = le.classes_.tolist()
            bisect.insort_left(cats, unknown_value)

            # redefine cats on le
            le.classes_ = np.asarray(cats)

            encoders[col] = le

        le = encoders[col]
        df[col] = df[col].map(lambda x: unknown_value if x not in le.classes_ else x)
        df[col] = le.transform(df[col].values)

    return df, encoders


def pickle_obj(path, obj):
    with open(path, 'wb') as f:
        pickle.dump(obj, f)


if __name__ == '__main__':
    log_fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(level=logging.INFO, format=log_fmt)

    # not used in this stub but often useful for finding various files
    project_dir = Path(__file__).resolve().parents[2]

    # find .env automagically by walking up directories until it's found, then
    # load up the .env entries as environment variables
    load_dotenv(find_dotenv())

    main()



# -*- coding: utf-8 -*-
import click
import logging
from pathlib import Path
from dotenv import find_dotenv, load_dotenv

import pandas as pd
# import seaborn as sns
import plotly
# import chart_studio.plotly as py
import plotly.figure_factory as ff
# import sys
import missingno as msno
import matplotlib.pyplot as plt


@click.command()
@click.argument('input_filepath', type=click.Path(exists=True), default='data/raw/')
@click.argument('output_filepath', type=click.Path(), default='models/')
@click.argument('report_filepath', type=click.Path(), default='reports/')
@click.argument('figure_filepath', type=click.Path(), default='reports/figures/')
@click.argument('eda_filepath', type=click.Path(), default='reports/eda/')
def main(input_filepath, output_filepath, report_filepath, figure_filepath, eda_filepath):
    """ Runs viz scripts to turn raw data from (../raw) into
        to eda diagrams (../reports/eda)
    """
    logger = logging.getLogger(__name__)

    DATADICT_FILEPATH = Path.cwd().joinpath(input_filepath).joinpath('WiDS Datathon 2020 Dictionary.csv')
    RAW_DF_FILEPATH = Path.cwd().joinpath(input_filepath).joinpath('training_v2.csv')
    INPUT_DIR = Path.cwd().joinpath(input_filepath)
    logger.info(f'modelling with data in {INPUT_DIR.name}')

    logger.info('Load Data')
    datadict = pd.read_csv(DATADICT_FILEPATH)
    train = pd.read_csv(RAW_DF_FILEPATH)

    logger.info('generate types')
    continuous_cols = list(
        list(datadict[datadict['Data Type'] == 'integer']['Variable Name'].unique())
        + list(datadict[datadict['Data Type'] == 'numeric']['Variable Name'].unique())
    )
    # categorical_cols = list(datadict[datadict['Data Type'] == 'string']['Variable Name'].unique())
    # binary_cols = list(datadict[datadict['Data Type'] == 'binary']['Variable Name'].unique())
    # target_cols = 'hospital_death'

    logger.info('generating kde plots')
    for col in continuous_cols:
        try:
            logger.info(f'generating {col} kde')
            plot_data = list()
            plot_labels = list()
            for hospital_id in train['hospital_id'].unique():

                subset = train[train['hospital_id'] == hospital_id]

                tmp = subset[col].fillna(0)
                if 0 < tmp.shape[0]:
                    plot_data.append(tmp)
                    plot_labels.append(str(hospital_id))

            fig = ff.create_distplot(plot_data, plot_labels, show_hist=False, show_rug=False)
            fig.update_layout(title_text=f'{col} curves')
            plotly.offline.plot(fig, filename=str(Path.cwd().joinpath(
                eda_filepath).joinpath('kde/').joinpath(f'kde-{col}.html')), auto_open=False)
        except Exception as e:
            logger.error(e, exc_info=True)

    logger.info('generating missingno figures')
    msno.matrix(train.sample(min(1000, len(train))), figsize=(100, 100), sort='ascending', labels=True)
    plt.savefig(str(Path.cwd().joinpath(eda_filepath).joinpath('missingno/').joinpath(f'matrix-all.png')))

    msno.bar(train.sample(min(1000, len(train))), figsize=(100, 100), labels=True)
    plt.savefig(str(Path.cwd().joinpath(eda_filepath).joinpath('missingno/').joinpath(f'bar-all.png')))

    msno.heatmap(train.sample(min(1000, len(train))), figsize=(100, 100), labels=True)
    plt.savefig(str(Path.cwd().joinpath(eda_filepath).joinpath('missingno/').joinpath(f'heatmap-all.png')))
    # for hospital_id in train['hospital_id'].unique():
    #     plt.close(fig='all')
    #     logger.info(f'plotting msno for {hospital_id}')
    #     subset = train[train['hospital_id'] == hospital_id]

    #     msno.matrix(subset.sample(min(1000, len(subset))), figsize=(100, 100), sort='ascending', labels=True)
    #     plt.savefig(str(Path.cwd().joinpath(eda_filepath).joinpath('missingno/').joinpath(f'matrix-{hospital_id}.png')))

    #     msno.bar(subset.sample(min(1000, len(subset))), figsize=(100, 100), labels=True)
    #     plt.savefig(str(Path.cwd().joinpath(eda_filepath).joinpath('missingno/').joinpath(f'bar-{hospital_id}.png')))

    #     msno.heatmap(subset.sample(min(1000, len(subset))), figsize=(100, 100), labels=True)
    #     plt.savefig(str(Path.cwd().joinpath(eda_filepath).joinpath('missingno/').joinpath(f'heatmap-{hospital_id}.png')))


if __name__ == '__main__':
    log_fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(level=logging.INFO, format=log_fmt)

    # not used in this stub but often useful for finding various files
    project_dir = Path(__file__).resolve().parents[2]

    # find .env automagically by walking up directories until it's found, then
    # load up the .env entries as environment variables
    load_dotenv(find_dotenv())

    main()



# -*- coding: utf-8 -*-
import click
import logging
from pathlib import Path
from dotenv import find_dotenv, load_dotenv
import zipfile


@click.command()
@click.argument('input_filepath', type=click.Path(exists=True), default='data/external/widsdatathon2020.zip')
@click.argument('output_filepath', type=click.Path(exists=True), default='data/raw/')
def unzip_dataset(input_filepath, output_filepath):
    """ Unzip a dataset from external data (data/external) into
        the raw staging directory (data/raw).
    """
    logger = logging.getLogger(__name__)
    logger.info(f'unzipping file {input_filepath} to output filepath {output_filepath}')

    DATASET = Path.cwd().joinpath(input_filepath)

    with zipfile.ZipFile(DATASET, 'r') as zip_ref:
        zip_ref.extractall(output_filepath)


if __name__ == '__main__':
    log_fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(level=logging.INFO, format=log_fmt)

    # not used in this stub but often useful for finding various files
    project_dir = Path(__file__).resolve().parents[2]

    # find .env automagically by walking up directories until it's found, then
    # load up the .env entries as environment variables
    load_dotenv(find_dotenv())

    unzip_dataset()



# -*- coding: utf-8 -*-
import click
import logging
from pathlib import Path
from dotenv import find_dotenv, load_dotenv
import pandas as pd


def make_dataset(input_csv_filepath, output_dirpath):
    """ Runs data processing scripts to transform raw data from (../raw) into
        cleaned and data-defacto view ready to be analyzed/feature-engineered/encoded
        (saved in ../interim).
    """
    logger = logging.getLogger(__name__)
    logger.info(f'making cleaned dataset from {input_csv_filepath}')

    # Get data
    logger.info('getting data...')
    df = pd.read_csv(Path.cwd().joinpath(input_csv_filepath))

    # Get Col Types
    continuous_cols = [x for x in df.columns if x in ['encounter_id', 'hospital_id', 'patient_id', 'icu_id', 'gcs_eyes_apache', 'gcs_motor_apache', 'gcs_verbal_apache', 'age', 'height', 'pre_icu_los_days', 'weight', 'albumin_apache', 'bilirubin_apache', 'bun_apache', 'creatinine_apache', 'fio2_apache', 'glucose_apache', 'heart_rate_apache', 'hematocrit_apache', 'map_apache', 'paco2_apache', 'paco2_for_ph_apache', 'pao2_apache', 'ph_apache', 'resprate_apache', 'sodium_apache', 'temp_apache', 'urineoutput_apache', 'wbc_apache', 'd1_diasbp_invasive_max', 'd1_diasbp_invasive_min', 'd1_diasbp_max', 'd1_diasbp_min', 'd1_diasbp_noninvasive_max', 'd1_diasbp_noninvasive_min', 'd1_heartrate_max', 'd1_heartrate_min', 'd1_mbp_invasive_max', 'd1_mbp_invasive_min', 'd1_mbp_max', 'd1_mbp_min', 'd1_mbp_noninvasive_max', 'd1_mbp_noninvasive_min', 'd1_resprate_max', 'd1_resprate_min', 'd1_spo2_max', 'd1_spo2_min', 'd1_sysbp_invasive_max', 'd1_sysbp_invasive_min', 'd1_sysbp_max', 'd1_sysbp_min', 'd1_sysbp_noninvasive_max', 'd1_sysbp_noninvasive_min', 'd1_temp_max', 'd1_temp_min', 'h1_diasbp_invasive_max', 'h1_diasbp_invasive_min', 'h1_diasbp_max', 'h1_diasbp_min', 'h1_diasbp_noninvasive_max', 'h1_diasbp_noninvasive_min', 'h1_heartrate_max', 'h1_heartrate_min', 'h1_mbp_invasive_max', 'h1_mbp_invasive_min', 'h1_mbp_max', 'h1_mbp_min', 'h1_mbp_noninvasive_max', 'h1_mbp_noninvasive_min', 'h1_resprate_max', 'h1_resprate_min', 'h1_spo2_max', 'h1_spo2_min', 'h1_sysbp_invasive_max', 'h1_sysbp_invasive_min', 'h1_sysbp_max', 'h1_sysbp_min', 'h1_sysbp_noninvasive_max', 'h1_sysbp_noninvasive_min', 'h1_temp_max', 'h1_temp_min', 'd1_albumin_max', 'd1_albumin_min', 'd1_bilirubin_max', 'd1_bilirubin_min', 'd1_bun_max', 'd1_bun_min', 'd1_calcium_max', 'd1_calcium_min', 'd1_creatinine_max', 'd1_creatinine_min', 'd1_glucose_max', 'd1_glucose_min', 'd1_hco3_max', 'd1_hco3_min', 'd1_hemaglobin_max', 'd1_hemaglobin_min', 'd1_hematocrit_max', 'd1_hematocrit_min', 'd1_inr_max', 'd1_inr_min', 'd1_lactate_max', 'd1_lactate_min', 'd1_platelets_max', 'd1_platelets_min', 'd1_potassium_max', 'd1_potassium_min', 'd1_sodium_max', 'd1_sodium_min', 'd1_wbc_max', 'd1_wbc_min', 'h1_albumin_max', 'h1_albumin_min', 'h1_bilirubin_max', 'h1_bilirubin_min', 'h1_bun_max', 'h1_bun_min', 'h1_calcium_max', 'h1_calcium_min', 'h1_creatinine_max', 'h1_creatinine_min', 'h1_glucose_max', 'h1_glucose_min', 'h1_hco3_max', 'h1_hco3_min', 'h1_hemaglobin_max', 'h1_hemaglobin_min', 'h1_hematocrit_max', 'h1_hematocrit_min', 'h1_inr_max', 'h1_inr_min', 'h1_lactate_max', 'h1_lactate_min', 'h1_platelets_max', 'h1_platelets_min', 'h1_potassium_max', 'h1_potassium_min', 'h1_sodium_max', 'h1_sodium_min', 'h1_wbc_max', 'h1_wbc_min', 'd1_arterial_pco2_max', 'd1_arterial_pco2_min', 'd1_arterial_ph_max', 'd1_arterial_ph_min', 'd1_arterial_po2_max', 'd1_arterial_po2_min', 'd1_pao2fio2ratio_max', 'd1_pao2fio2ratio_min', 'h1_arterial_pco2_max', 'h1_arterial_pco2_min', 'h1_arterial_ph_max', 'h1_arterial_ph_min', 'h1_arterial_po2_max', 'h1_arterial_po2_min', 'h1_pao2fio2ratio_max', 'h1_pao2fio2ratio_min', 'apache_4a_hospital_death_prob', 'apache_4a_icu_death_prob', 'bmi', 'apache_2_diagnosis', 'apache_3j_diagnosis']]  # noqa
    categorical_cols = [x for x in df.columns if x in ['ethnicity', 'gender', 'hospital_admit_source', 'icu_admit_source', 'icu_stay_type', 'icu_type', 'apache_3j_bodysystem', 'apache_2_bodysystem']]  # noqa
    binary_cols = [x for x in df.columns if x in ['elective_surgery', 'readmission_status', 'apache_post_operative', 'arf_apache', 'gcs_unable_apache', 'intubated_apache', 'ventilated_apache', 'aids', 'cirrhosis', 'diabetes_mellitus', 'hepatic_failure', 'immunosuppression', 'leukemia', 'lymphoma', 'solid_tumor_with_metastasis']]  # noqa
    target_col = [x for x in df.columns if x in ['hospital_death']]

    logger.info(f'Target cols: {target_col}')
    logger.info(f'Continuous cols: {continuous_cols}')
    logger.info(f'Categorical cols: {categorical_cols}')
    logger.info(f'Binary cols: {binary_cols}')

    logger.info(f'Masking columns')
    df = df[target_col + continuous_cols + categorical_cols + binary_cols]
    logger.info(f'Dataframe has these cols {list(df.columns)}')

    # Typify
    logger.info('typifying...')
    if 0 < len(continuous_cols):
        df[continuous_cols] = df[continuous_cols].astype('float32')
    if 0 < len(categorical_cols):
        df[categorical_cols] = df[categorical_cols].astype('str').astype('category')
    if 0 < len(binary_cols):
        df[binary_cols] = df[binary_cols].astype('str').astype('category')
    if 0 < len(target_col):
        df[target_col] = df[target_col].astype('str').astype('category')

    # Export
    output_filename = Path.cwd().joinpath(input_csv_filepath).stem + '.feather'
    df.to_feather(Path.cwd().joinpath(output_dirpath).joinpath(output_filename))


@click.command()
@click.argument('input_csv_filepath', type=click.Path(exists=True), default='data/raw/training_v2.csv')
@click.argument('output_dirpath', type=click.Path(exists=True), default='data/interim/')
def main(input_csv_filepath, output_dirpath):
    make_dataset(input_csv_filepath, output_dirpath)


if __name__ == '__main__':
    log_fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(level=logging.INFO, format=log_fmt)

    # not used in this stub but often useful for finding various files
    project_dir = Path(__file__).resolve().parents[2]

    # find .env automagically by walking up directories until it's found, then
    # load up the .env entries as environment variables
    load_dotenv(find_dotenv())

    main()



# -*- coding: utf-8 -*-
import click
import logging
from pathlib import Path
from dotenv import find_dotenv, load_dotenv
import pandas as pd
from catboost import CatBoostClassifier


def predict_model(inference_filepath, model_filepath, output_filepath):
    """ Runs modelling scripts to turn preprocessed data from (../processed) into
        to a model (../models)
    """
    logger = logging.getLogger(__name__)

    logger.info(f'loading model {model_filepath}')
    model = CatBoostClassifier().load_model(str(Path.cwd().joinpath(model_filepath)))

    logger.info(f'loading inference data {inference_filepath}')
    df = pd.read_feather(Path.cwd().joinpath(inference_filepath))

    # Store the encounter ids, before removing them for training
    arr = df['encounter_id']

    logger.info(f'inference data has cols: {df.columns}')
    logger.info(f'masking inference data to cols: {model.feature_names_}')
    X = df[model.feature_names_]

    logger.info('making predictions')
    y_proba = model.predict_proba(X)
    y_proba_death = y_proba[:, 1]

    y = pd.DataFrame(y_proba_death, columns=['hospital_death']).astype('float32')

    logger.info('persisting predictions')
    # arr = scalers['encounter_id'].inverse_transform(X['encounter_id'])
    X_encounter_id = round(pd.DataFrame(arr, columns=['encounter_id']))  # round for numerical errs
    X_encounter_id = X_encounter_id.astype('int32')

    output_filename = Path.cwd().joinpath(inference_filepath).stem + '.csv'
    pd.concat([X_encounter_id, y], axis=1).to_csv(Path.cwd().joinpath(output_filepath).joinpath(output_filename), index=False)


@click.command()
@click.argument('inference_filepath', type=click.Path(exists=True), default='data/processed/unlabeled_encoded.feather')
@click.argument('model_filepath', type=click.Path(exists=True), default='models/model.dump')
@click.argument('output_filepath', type=click.Path(), default='data/predictions/')
def main(inference_filepath, model_filepath, output_filepath):
    predict_model(inference_filepath, model_filepath, output_filepath)


if __name__ == '__main__':
    log_fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(level=logging.INFO, format=log_fmt)

    # not used in this stub but often useful for finding various files
    project_dir = Path(__file__).resolve().parents[2]

    # find .env automagically by walking up directories until it's found, then
    # load up the .env entries as environment variables
    load_dotenv(find_dotenv())

    main()



# -*- coding: utf-8 -*-
import click
import logging
from pathlib import Path
from dotenv import find_dotenv, load_dotenv
import pandas as pd
from sklearn.model_selection import train_test_split
import numpy as np


def stratify_dataset(input_filepath, output_filepath):
    """ Runs data stratifying scripts to form
        train, validation and test sets.
    """
    logger = logging.getLogger(__name__)
    logger.info(f'stratifying file {input_filepath}')

    df = pd.read_feather(Path.cwd().joinpath(input_filepath))

    # Splitting
    strat_cols = ['hospital_death']
    logger.info(f'stratifying on cols: {strat_cols} and hospital_id')

    train = pd.DataFrame()
    val = pd.DataFrame()
    test = pd.DataFrame()

    for hospital_id in df['hospital_id'].unique():
        subset = df[df['hospital_id'] == hospital_id]

        if 10 < len(subset):
            classes, y_indices = np.unique(
                subset[strat_cols],
                return_inverse=True
            )
            class_counts = np.bincount(y_indices)

            if 2 > np.min(class_counts):
                tmp_train, tmp_test = train_test_split(
                    subset,
                    test_size=0.1,
                    random_state=0
                )
                tmp_train, tmp_val = train_test_split(
                    tmp_train,
                    test_size=0.1,
                    random_state=0
                )
            else:
                tmp_train, tmp_test = train_test_split(
                    subset,
                    test_size=0.1,
                    random_state=0,
                    stratify=subset[strat_cols]
                )
                tmp_train, tmp_val = train_test_split(
                    tmp_train,
                    test_size=0.1,
                    random_state=0,
                    stratify=tmp_train[strat_cols]
                )

            train = train.append(tmp_train)
            val = val.append(tmp_val)
            test = test.append(tmp_test)

    output_filename = Path.cwd().joinpath(input_filepath).stem
    train.reset_index(drop=True).to_feather(Path.cwd().joinpath(output_filepath).joinpath(output_filename + '_train.feather'))
    val.reset_index(drop=True).to_feather(Path.cwd().joinpath(output_filepath).joinpath(output_filename + '_val.feather'))
    test.reset_index(drop=True).to_feather(Path.cwd().joinpath(output_filepath).joinpath(output_filename + '_test.feather'))


@click.command()
@click.argument('input_filepath', type=click.Path(exists=True), default='data/interim/training_v2.feather')
@click.argument('output_filepath', type=click.Path(exists=True), default='data/interim/')
def main(input_filepath, output_filepath):
    stratify_dataset(input_filepath, output_filepath)


if __name__ == '__main__':
    log_fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(level=logging.INFO, format=log_fmt)

    # not used in this stub but often useful for finding various files
    project_dir = Path(__file__).resolve().parents[2]

    # find .env automagically by walking up directories until it's found, then
    # load up the .env entries as environment variables
    load_dotenv(find_dotenv())

    main()



# -*- coding: utf-8 -*-
import click
import logging
from pathlib import Path
from pathlib import PurePath
from dotenv import find_dotenv, load_dotenv
from wids_datathon_2020.data.make_dataset import make_dataset
from wids_datathon_2020.data.encode_dataset import encode_dataset
from wids_datathon_2020.models.predict_model import predict_model
import tempfile
import pandas as pd


def inference(dataset_filepath: str):
    """ Runs modelling scripts to turn preprocessed data from (../processed) into
        to a model (../models)
    """
    logger = logging.getLogger(__name__)
    logger.info(f'learning from file {dataset_filepath}...')

    dataset_filename = Path.cwd().joinpath(dataset_filepath).stem

    make_dataset(dataset_filepath, 'data/interim/')
    encode_dataset('data/interim/' + dataset_filename + '.feather', 'data/processed/', 'models/', False)
    predict_model('data/processed/' + dataset_filename + '_encoded.feather', 'models/model.dump', 'data/predictions/')


def inference_sample(samples):
    logger = logging.getLogger(__name__)
    logger.info(f'performing inference on samples {samples}...')

    preds = dict()
    with tempfile.NamedTemporaryFile(suffix='.csv', mode='wt') as inference_file:
        inputs = pd.DataFrame.from_dict(
            samples,
            orient='index'
        )
        inputs.to_csv(inference_file, index=False)
        inference_file.seek(0)

        inference_file_stem = PurePath(inference_file.name).stem
        inference(inference_file.name)
        encoded = pd.read_feather('data/processed/' + inference_file_stem + '_encoded.feather').to_dict(orient='record')
        preds = pd.read_csv(
            f'./data/predictions/{inference_file_stem}_encoded.csv'
        )[['encounter_id', 'hospital_death']].to_dict(orient='record')
    return encoded, preds


@click.command()
@click.argument('dataset_filepath', type=click.Path(exists=True), default='data/raw/training_v2.csv')
def main(dataset_filepath):
    inference(dataset_filepath)


if __name__ == '__main__':
    log_fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(level=logging.INFO, format=log_fmt)

    # not used in this stub but often useful for finding various files
    project_dir = Path(__file__).resolve().parents[2]

    # find .env automagically by walking up directories until it's found, then
    # load up the .env entries as environment variables
    load_dotenv(find_dotenv())

    main()



# -*- coding: utf-8 -*-
import click
import logging
from pathlib import Path
from dotenv import find_dotenv, load_dotenv
import pandas as pd
import pickle
import numpy as np
from catboost import CatBoostClassifier
from catboost import Pool
# from sklearn.metrics import confusion_matrix,
from sklearn.metrics import roc_curve, auc, accuracy_score, f1_score, classification_report
# import pprint
import json
# import shap
# import matplotlib.pyplot as plt


def train_model(train_filepath, val_filepath, test_filepath, output_filepath, report_filepath, figure_filepath):
    """ Runs modelling scripts to turn preprocessed data from (../processed) into
        to a model (../models)
    """
    logger = logging.getLogger(__name__)
    target_col = 'hospital_death'

    logger.info(f'loading train data in {train_filepath}')
    X_train = pd.read_feather(Path.cwd().joinpath(train_filepath))
    y_train = X_train.pop(target_col)

    logger.info(f'loading val data in {val_filepath}')
    X_val = pd.read_feather(Path.cwd().joinpath(val_filepath))
    y_val = X_val.pop(target_col)

    logger.info(f'loading test data in {test_filepath}')
    X_test = pd.read_feather(Path.cwd().joinpath(test_filepath))
    y_test = X_test.pop(target_col)

    exclude_cols = ['apache_4a_hospital_death_prob', 'apache_3j_diagnosis', 'apache_4a_icu_death_prob',
                    'apache_2_diagnosis', 'apache_3j_bodysystem', 'apache_2_bodysystem', 'apache_post_operative']
    include_cols = ['age', 'ventilated_apache', 'elective_surgery',
                    'gcs_verbal_apache',  'gcs_motor_apache', 'd1_spo2_min', 'icu_id']

    categorical_cols = X_train.select_dtypes(include='category').columns
    categorical_cols = [x for x in categorical_cols if x not in exclude_cols and (0 == len(include_cols) or x in include_cols)]
    logger.info(f'using categorical cols: {categorical_cols}')

    numerical_cols = X_train.select_dtypes(include='number').columns
    numerical_cols = [x for x in numerical_cols if x not in exclude_cols and (0 == len(include_cols) or x in include_cols)]
    logger.info(f'using numerical cols: {numerical_cols}')

    X_train = X_train[categorical_cols + numerical_cols]
    X_val = X_val[categorical_cols + numerical_cols]
    X_test = X_test[categorical_cols + numerical_cols]

    logger.info('modelling')
    model_args = {
        # 'custom_loss': ['AUC'],
        'eval_metric': 'AUC',
        'random_seed': 42,
        'logging_level': 'Silent',
        'use_best_model': True
    }

    model = CatBoostClassifier(**model_args)
    model.fit(
        Pool(X_train, y_train, cat_features=categorical_cols),
        eval_set=Pool(X_val, y_val, cat_features=categorical_cols),
        logging_level='Verbose',
        plot=False
    )

    y_val_preds = model.predict(X_val)
    y_val_proba = model.predict_proba(X_val)
    y_val_proba_death = y_val_proba[:, 1]
    val_results = results(y_val, y_val_preds, y_val_proba_death)
    logger.info(f'val results: {val_results}')

    y_test_preds = model.predict(X_test)
    y_test_proba = model.predict_proba(X_test)
    y_test_proba_death = y_test_proba[:, 1]
    test_results = results(y_test, y_test_preds, y_test_proba_death)
    logger.info(f'test results: {test_results}')

    logger.info('generating feature importances')
    fi_args = {
        'model': model,
        'cat_cols': categorical_cols,
        'continuous_cols': numerical_cols,
        'figure_filepath': figure_filepath,
        'report_filepath': report_filepath,
        'output_filepath': output_filepath
    }
    fi_args['X'] = X_train
    fi_args['y'] = y_train
    fi_args['file_tag'] = 'train'
    logger.info('generating feature importances - train')
    calc_feature_importances(**fi_args)

    fi_args['X'] = X_val
    fi_args['y'] = y_val
    fi_args['file_tag'] = 'val'
    logger.info('generating feature importances - val')
    calc_feature_importances(**fi_args)

    fi_args['X'] = X_test
    fi_args['y'] = y_test
    fi_args['file_tag'] = 'test'
    logger.info('generating feature importances - test')
    calc_feature_importances(**fi_args)

    logger.info('dumping data and saving model')
    dump_results(Path.cwd().joinpath(report_filepath).joinpath('val-results.txt'), val_results)
    dump_results(Path.cwd().joinpath(report_filepath).joinpath('test-results.txt'), test_results)

    model.save_model(str(Path.cwd().joinpath(output_filepath).joinpath('model.dump')))


def read_obj(path):
    with open(path, 'rb') as f:
        return pickle.load(f)
    return None


def results(y_true, y_pred, y_proba):
    results = {}
    results['accuracy'] = accuracy_score(y_true, y_pred)
    results['f1'] = f1_score(y_true, y_pred)
    fpr, tpr, thresh = roc_curve(y_true, y_proba)
    results['auc'] = auc(fpr, tpr)
    results['classification_report'] = classification_report(y_true, y_pred, output_dict=True)
    return results


def dump_results(path, results_dict):
    with open(path, 'w') as f:
        f.write(json.dumps(results_dict))


def pickle_obj(path, obj):
    with open(path, 'wb') as f:
        pickle.dump(obj, f)


def calc_feature_importances(
    X,
    y,
    file_tag,
    model,
    cat_cols,
    continuous_cols,
    figure_filepath,
    report_filepath,
    output_filepath
):

    logging.info('computing shap values')
    shap_values = model.get_feature_importance(Pool(X, label=y, cat_features=cat_cols), type='ShapValues')
    # expected_values = shap_values[0, -1]
    shap_values = shap_values[:, :-1]

    # logging.info('computing summary bar plot')
    # fig = shap.summary_plot(shap_values, X, plot_type='bar', max_display=200, show=False)
    # plt.savefig(Path.cwd().joinpath(figure_filepath).joinpath(f'{file_tag}-summary_plot_bar.png'), bbox_inches='tight')

    logging.info('computing feature importances')
    vals = np.abs(shap_values).mean(0)
    feature_importance = pd.DataFrame(list(zip(X.columns, vals)), columns=['col_name', 'feature_importance_vals'])
    feature_importance.sort_values(by=['feature_importance_vals'], ascending=False, inplace=True)
    feature_importance.to_csv(Path.cwd().joinpath(report_filepath).joinpath(
        f'{file_tag}-feature-importances.csv'), index=False)

    # logging.info('computing summary plot')
    # fig = shap.summary_plot(shap_values, X, show=False)
    # plt.savefig(Path.cwd().joinpath(figure_filepath).joinpath(f'{file_tag}-summary_plot.png'), bbox_inches='tight')

    # logging.info('computing summary decision plot')
    # fig = shap.decision_plot(expected_values, shap_values[:2000], feature_names=[x for x in X.columns], show=False)
    # plt.savefig(Path.cwd().joinpath(figure_filepath).joinpath(f'{file_tag}-decision_plot.png'), bbox_inches='tight')

    # fi = model.get_feature_importance(
    #     Pool(X, label=y, cat_features=cat_cols),
    #     type="Interaction"
    # )
    # fi_new = []
    # for k,item in enumerate(fi):
    #     first = X.dtypes.index[fi[k][0]]
    #     second = X.dtypes.index[fi[k][1]]
    #     if first != second:
    #         fi_new.append([first + "_" + second, fi[k][2]])
    # feature_score = pd.DataFrame(fi_new,columns=['Feature-Pair','Score'])
    # feature_score =
    #   feature_score.sort_values(
    #       by='Score',
    #       ascending=False,
    #       inplace=False,
    #       kind='quicksort',
    #       na_position='last'
    #   )
    # ax = feature_score.plot('Feature-Pair', 'Score', kind='bar', color='c')
    # ax.set_title("Pairwise Feature Importance", fontsize = 14)
    # ax.set_xlabel("features Pair")
    # plt.savefig(
    # Path.cwd()
    #   .joinpath(figure_filepath)
    #   .joinpath(f'{file_tag}-pairwise-feature-importances.png'),
    # bbox_inches='tight',
    # figsize=(800, 10))

    # for feature in feature_importance.head(40)['col_name']:
    #     logging.info(f'generating {feature} - feature statistics')
    #     if feature in continuous_cols:
    #         for prediction_type in ['Probability', 'Class']:
    #             model.calc_feature_statistics(
    #                 X,
    #                 y,Í
    #                 feature,
    #                 plot=False,
    #                 prediction_type=prediction_type,
    #                 plot_file=Path.cwd().joinpath(figure_filepath).joinpath(f'{file_tag}-feature-statistics-{feature}-{prediction_type}.html')
    #             )

    # pickle_obj(Path.cwd().joinpath(output_filepath).joinpath(f'{file_tag}-expected-values.pickle'), expected_values)
    # pickle_obj(Path.cwd().joinpath(output_filepath).joinpath(f'{file_tag}-shap-values.pickle'), shap_values)


@click.command()
@click.argument('train_filepath', type=click.Path(exists=True), default='data/processed/training_v2_train_encoded.feather')
@click.argument('val_filepath', type=click.Path(exists=True), default='data/processed/training_v2_val_encoded.feather')
@click.argument('test_filepath', type=click.Path(exists=True), default='data/processed/training_v2_test_encoded.feather')
@click.argument('output_filepath', type=click.Path(exists=True), default='models/')
@click.argument('report_filepath', type=click.Path(exists=True), default='reports/')
@click.argument('figure_filepath', type=click.Path(exists=True), default='reports/figures/')
def main(train_filepath, val_filepath, test_filepath, output_filepath, report_filepath, figure_filepath):
    train_model(train_filepath, val_filepath, test_filepath, output_filepath, report_filepath, figure_filepath)


if __name__ == '__main__':
    log_fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(level=logging.INFO, format=log_fmt)

    # not used in this stub but often useful for finding various files
    project_dir = Path(__file__).resolve().parents[2]

    # find .env automagically by walking up directories until it's found, then
    # load up the .env entries as environment variables
    load_dotenv(find_dotenv())

    main()



from setuptools import find_packages, setup

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='wids-datathon-2020',
    version='0.1.4',
    license='MIT',
    author='Iain Wong',
    author_email='iainwong@outlook.com',
    description='The challenge is to create a model that uses data from the first 24 hours of intensive care to predict patient survival. (Kaggle Proj) https://www.kaggle.com/c/widsdatathon2020/overview',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/iainwo/kaggle/tree/master/wids-datathon-2020',
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'pandas',
        'python-dotenv',
        'Click',
        'pyarrow',
        'numpy',
        'scikit-learn',
    ],
    python_requires='>=3.6',
)



# -*- coding: utf-8 -*-
import click
import logging
from pathlib import Path
from dotenv import find_dotenv, load_dotenv
from wids_datathon_2020.data.make_dataset import make_dataset
from wids_datathon_2020.data.stratify_dataset import stratify_dataset
from wids_datathon_2020.data.encode_dataset import encode_dataset
from wids_datathon_2020.models.train_model import train_model


def learn(dataset_filepath: str):
    """ Runs modelling scripts to turn preprocessed data from (../processed) into
        to a model (../models)
    """
    logger = logging.getLogger(__name__)
    logger.info(f'learning from file {dataset_filepath}...')

    dataset_filename = Path.cwd().joinpath(dataset_filepath).stem

    make_dataset(dataset_filepath, 'data/interim/')
    stratify_dataset('data/interim/' + dataset_filename + '.feather', 'data/interim/')
    encode_dataset('data/interim/' + dataset_filename + '_train.feather', 'data/processed/', 'models/', True)
    encode_dataset('data/interim/' + dataset_filename + '_val.feather', 'data/processed/', 'models/', False)
    encode_dataset('data/interim/' + dataset_filename + '_test.feather', 'data/processed/', 'models/', False)
    train_model(
        'data/processed/' + dataset_filename + '_train_encoded.feather',
        'data/processed/' + dataset_filename + '_val_encoded.feather',
        'data/processed/' + dataset_filename + '_test_encoded.feather',
        'models/',
        'reports/',
        'reports/figures/'
    )


@click.command()
@click.argument('dataset_filepath', type=click.Path(exists=True), default='data/raw/training_v2.csv')
def main(dataset_filepath):
    learn(dataset_filepath)


if __name__ == '__main__':
    log_fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(level=logging.INFO, format=log_fmt)

    # not used in this stub but often useful for finding various files
    project_dir = Path(__file__).resolve().parents[2]

    # find .env automagically by walking up directories until it's found, then
    # load up the .env entries as environment variables
    load_dotenv(find_dotenv())

    main()
